// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'ngMessages', 'uiGmapgoogle-maps', 'ngCordova', 'ngStorage', 'satellizer', 'truncate', 'ion-google-place', 'starter.services', 'starter.filters', 'starter.directives', 'starter.controllers'])

.run(function($ionicPlatform, $q, $rootScope, $state) {
  var viewLoadDefer = $q.defer();
  var viewLoadUnbind = $rootScope.$on('$ionicView.enter', function() {
    viewLoadDefer.resolve();
    viewLoadUnbind();
  })

  $rootScope.$on('$stateChangeError', function(event, toState, toParams, fromState, fromParams, error) {
    // ui-router will try to go to the previous view by default
    // in some cases there will be no previous view, just navigate to the otherwise route
    if(!fromState || !fromState.name) {
      event.preventDefault();
      $state.go('app.home');
    }
  })

  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if(window.cordova && window.cordova.plugins.Keyboard) {
      // cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
      cordova.plugins.Keyboard.disableScroll(true);
    }

    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      // StatusBar.styleDefault();
      StatusBar.styleLightContent()
    }
  });

  // Need to setup this way as either ionicPlatform.ready or $ionicView.enter can fire first, but we need both to be ready
  $q.all([
    viewLoadDefer.promise,
    $ionicPlatform.ready()
  ]).then(function() {
    if(navigator && navigator.splashscreen) {
      navigator.splashscreen.hide()
    }
    $rootScope.$broadcast('app:visible');
  });
})

.config(['uiGmapGoogleMapApiProvider', function(GoogleMapApi) {
// TODO: Update
  GoogleMapApi.configure({
    key: 'AIzaSyCHnJHHjRzC9RWaxE7iLJsOvr9v2pZF4fg',
    v: '3.24',
    libraries: 'places'
  });
}])

.config(function($authProvider) {
  // Satellizer configuration that specifies which API
  // route the JWT should be retrieved from

  // TODO: Update
  $authProvider.baseUrl = 'https://worldmarkapp.com/api-v2/1/';
  // $authProvider.baseUrl = 'http://sandbox.v2.reaswmsp.icmsdemo.com/api/v1/';
  $authProvider.loginUrl = 'authenticate';
  $authProvider.signupUrl = 'signup';

  $authProvider.httpInterceptor = true;
})

.factory('httpResponseErrorInterceptor', function($q, $injector, $rootScope, AUTH_EVENTS, SatellizerShared, SatellizerConfig, API) {
  return {
    'responseError': function(response) {
      if(response.config && (response.config.bypassHttpResponseErrorInterceptor || response.config.skipAuthorization)) {
        return $q.reject(response);
      }

      var shouldRefresh = false;
      var originalResponse = response;

      // Check if token missing
      if(response.status === 400 && response.data.error == 'token_not_provided') {
        // Check if Satellizer didn't send the token because it had expired
        if(!SatellizerShared.isAuthenticated() && SatellizerShared.getToken()) {
          shouldRefresh = true;
        }
      }

      // Another way for a token to be expired
      if(response.status === 401 && response.data.error == 'token_expired') {
        shouldRefresh = true;
      }

      // Token is no good
      if(((response.status === 500 || response.status === 401) && response.data.error == 'token_invalid') || (response.status === 404 && response.data.error == 'user_not_found')) {
        // Forcibly log out the user
        $rootScope.$broadcast(AUTH_EVENTS.logout);
      }

      if(shouldRefresh) {
        var $http = $injector.get('$http');

        var headers = {};

        headers[SatellizerConfig.authHeader] = (function() {
          var token = SatellizerShared.getToken();
          if(SatellizerConfig.authHeader && SatellizerConfig.authToken) {
            token = SatellizerConfig.authToken + ' ' + token;
          }
          return token;
        })();

        var deferred = $q.defer();

        $http({
          url: API.url('authenticate/refresh'),
          method: 'POST',
          headers: headers,

          // We don't want to run this interceptor on the refresh request
          bypassHttpResponseErrorInterceptor: true
        })
        .then(function(response) {
          var refreshToken = response.headers(SatellizerConfig.authHeader);
          if(SatellizerConfig.authHeader && SatellizerConfig.authToken) {
            refreshToken = refreshToken.replace(SatellizerConfig.authToken + ' ', '');
          }
          SatellizerShared.setToken({access_token: refreshToken});

          // Retry original request
          deferred.resolve($http(originalResponse.config));
        })
        .catch(function(response) {
          if(response.status === 400 && response.data.error == 'token_expired') {
            // Forcibly log out the user
            $rootScope.$broadcast(AUTH_EVENTS.logout);
          } else {
            deferred.reject(originalResponse);
          }
        });

        return deferred.promise;
      } else {
        // Give up
        return $q.reject(response);
      }
    }
  };
})

.config(function($httpProvider) {
  $httpProvider.interceptors.push('httpResponseErrorInterceptor');
})

.config(function($stateProvider, $urlRouterProvider) {
  var requireDeviceLoad = function(DeviceService) {
    return DeviceService.setup();
  }

  var requireLogin = function($q, $auth, LoginModalService) {
    if(!$auth.getToken()) {
      return LoginModalService.show(true);
    } else {
      return $q.resolve(true);
    }
  }

  var requirePushNotifications = function(PushNotificationPermission) {
    return PushNotificationPermission.request().catch(function() {
      // We want the route to work even if we don't have push notifications
      return true;
    });
  }

  // We need to try to send out a pilot request
  // If we don't ensure only one request happens first, we can end up with multiple requests
  // all of which could end up trying to refresh the token and subsequently invalidate each other
  var tryProfileLoad = function($rootScope, $q, $auth, Profile) {
    // Try fetching the profile if the user is logged in
    if($auth.getToken()) {
      // Pilot is a request with a short timeout - we don't want the app to be loading for too long
      return Profile.pilot().$promise.then(function() {
        return $q.resolve(true);
      }, function() {
        return $q.resolve(true);
      });
    }
    return $q.resolve(true);
  }

  $stateProvider
  .state('app', {
    url: '/app',
    abstract: true,
    templateUrl: 'templates/menu.html',
    controller: 'AppCtrl',
    resolve: {
      requireDeviceLoad: requireDeviceLoad,
      tryProfileLoad: tryProfileLoad
    }
  })

  .state('app.external', {
    url: '/app/external',
    resolve: {
      open: function($stateParams) {
        if($stateParams.url) {
          window.open($stateParams.url, '_system');
        }
        throw 'Opening external link';
      }
    },
    params: {
      url: null
    }
  })

  .state('app.logout', {
    url: '/app/logout',
    resolve: {
      logout: function($auth, $rootScope, $ionicHistory, $state, AUTH_EVENTS) {
        $auth.logout();
        $rootScope.$broadcast(AUTH_EVENTS.logout);

        $ionicHistory.nextViewOptions({
          historyRoot: true,
          disableBack: true,
        });
        $ionicHistory.clearCache();
      }
    },
    onEnter: function($state, $timeout) {
      $timeout(function() {
        $state.go('app.home');
      });
    }
  })

  .state('app.home', {
    url: '/home',
    views: {
      'menuContent': {
        templateUrl: 'templates/home.html',
        controller: 'HomeCtrl'
      }
    }
  })

  .state('app.availability', {
    url: '/availability',
    views: {
      'menuContent': {
        templateUrl: 'templates/availability/index.html',
        controller: 'AvailabilityCtrl'
      }
    }
  })

  .state('app.availability.results', {
    url: '/:from_date/:to_date/:resorts',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/availability/results.html',
        controller: 'AvailabilityResultsCtrl'
      }
    }
  })

  .state('app.resorts', {
    url: '/resorts',
    abstract: true
  })

  .state('app.resorts.regions', {
    url: '/regions',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/region.html',
        controller: 'RegionCtrl'
      }
    }
  })

  .state('app.resorts.search', {
    url: '/search',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/search.html',
        controller: 'ResortSearchCtrl'
      }
    }
  })

  .state('app.resorts.listing', {
    url: '/search/:country/:state',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/listing.html',
        controller: 'ResortListingCtrl'
      }
    }
  })

  .state('app.resort', {
    url: '/resorts/{resortId}',
    views: {
      'menuContent': {
        templateUrl: 'templates/resorts/show.html',
        controller: 'ResortCtrl'
      }
    }
  })

  .state('app.resort.rooms', {
    url: '/rooms',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/show/rooms.html',
        controller: 'ResortRoomFacilitiesCtrl'
      }
    }
  })

  .state('app.resort.map', {
    url: '/map',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/show/map.html',
        controller: 'ResortMapCtrl'
      }
    },
    params: {
      coords: null
    }
  })

  .state('app.resort.facilities', {
    url: '/facilities',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/show/facilities.html',
        controller: 'ResortFacilitiesCtrl'
      }
    }
  })

  .state('app.resort.enquiry', {
    url: '/enquiry',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/show/enquiry.html',
        controller: 'ResortEnquiryCtrl'
      }
    }
  })

  .state('app.postcard', {
    url: '/postcard',
    abstract: true
  })

  .state('app.postcard.select-photo', {
    url: '/select-photo',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/postcard/1-select-photo.html',
        controller: 'PostcardSelectPhotoCtrl'
      }
    }
  })

  .state('app.postcard.choose-template', {
    url: '/choose-template',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/postcard/2-choose-template.html',
        controller: 'PostcardChooseTemplateCtrl'
      }
    }
  })

  .state('app.postcard.details', {
    url: '/details',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/postcard/3-details.html',
        controller: 'PostcardDetailsCtrl'
      }
    }
  })

  .state('app.postcard.output', {
    url: '/output',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/postcard/4-output.html',
        controller: 'PostcardOutputCtrl'
      }
    }
  })

  .state('app.notifications', {
    url: '/notifications/:resortId',
    views: {
      'menuContent': {
        templateUrl: 'templates/notifications.html',
        controller: 'NotificationListingCtrl'
      }
    },
    resolve: {
      requirePushNotifications: requirePushNotifications
    }
  })

  .state('app.profile', {
    url: '/profile',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/profile/index.html',
        controller: 'ProfileCtrl'
      }
    },
    resolve: {
      requireLogin: requireLogin
    }
  })

  .state('app.profile.booking-windows', {
    url: '/booking-windows',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/profile/booking-windows.html',
        controller: 'ProfileBookingWindowsCtrl'
      }
    },
    resolve: {
      requireLogin: requireLogin
    }
  })

  .state('app.holidays', {
    url: '/holidays',
    views: {
      'menuContent': {
        templateUrl: 'templates/holidays/index.html',
        controller: 'HolidayListingCtrl'
      }
    },
    resolve: {
      requireLogin
    }
  })

  .state('app.holidays.create', {
    url: '/create',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/holidays/create.html',
        controller: 'HolidayCreateCtrl'
      }
    },
    resolve: {
      requireLogin: requireLogin
    }
  })

  .state('app.holidays.show', {
    url: '/{holidayId}',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/holidays/show.html',
        controller: 'HolidayShowCtrl'
      }
    },
    resolve: {
      requireLogin: requireLogin
    }
  })

  .state('app.holidays.edit', {
    url: '/{holidayId}/edit',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/holidays/edit.html',
        controller: 'HolidayEditCtrl'
      }
    },
    resolve: {
      requireLogin: requireLogin
    }
  })

  .state('app.holidays.share', {
    url: '/{holidayId}/share',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/holidays/share.html',
        controller: 'HolidayShareCtrl'
      }
    },
    resolve: {
      profilePromise: function($q, $state, $ionicPopup, Profile) {
        return Profile.get().$promise.then(function(profile) {
          if(!profile.worldmark_journey_id) {
            $ionicPopup.confirm({
              title: 'Create a WorldMark Journey account?',
              template: '<p>In order to share your holiday you will need to signup for a WorldMark Journey account.</p><p><strong>Important:</strong> Once you have setup an account you will need to link it.</p>'
            }).then(function(res) {
              if(res) {
                window.open('https://worldmarkjourney.com/owner/register/', '_system');
                $state.go('app.profile');
              }
            });
            return $q.reject('no_worldmark_journey_id');
          }
          return profile;
        });
      },
      holidayPromise: function($stateParams, Holiday) {
        return Holiday.get($stateParams).$promise;
      }
    }
  })

  .state('app.articles', {
    url: '/articles',
    views: {
      'menuContent': {
        templateUrl: 'templates/articles/index.html',
        controller: 'ArticleListingCtrl'
      }
    }
  })

  .state('app.explore', {
    url: '/explore',
    views: {
      'menuContent': {
        templateUrl: 'templates/explore/index.html'
      }
    }
  })

  .state('app.explore.no-plan', {
    url: '/no-plan',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/explore/no-plan.html',
        controller: 'ExploreNoPlanCtrl'
      }
    }
  })

  .state('app.explore.no-plan.results', {
    url: '/results/:mood_activity/:mood_food/:mood_family/:mood_location',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/listing.html',
        controller: 'ExploreNoPlanResultsCtrl'
      }
    }
  })

  .state('app.explore.experience', {
    url: '/experience',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/explore/experience.html',
        controller: 'ExploreExperienceCtrl'
      }
    }
  })

  .state('app.explore.experience.results', {
    url: '/results/:facilities',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/resorts/listing.html',
        controller: 'ExploreExperienceResultsCtrl'
      }
    }
  })

  .state('app.planning', {
    url: '/planning',
    views: {
      'menuContent': {
        templateUrl: 'templates/planning.html',
        controller: 'PlanningCtrl'
      }
    }
  })

  .state('app.page', {
    url: '/page/:slug',
    views: {
      'menuContent': {
        templateUrl: 'templates/page.html',
        controller: 'PageCtrl'
      }
    }
  })

  .state('app.calculator', {
    url: '/calculator/{resortId}',
    views: {
      'menuContent': {
        templateUrl: 'templates/calculator/index.html',
        controller: 'CalculatorCtrl'
      }
    }
  })

  .state('app.calculator.results', {
    url: '/:room_type/:start_date/:end_date',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/calculator/results.html',
        controller: 'CalculatorResultsCtrl'
      }
    }
  })

  .state('app.calculator.enquiry', {
    url: '/enquiry',
    views: {
      'menuContent@app': {
        templateUrl: 'templates/calculator/enquiry.html',
        controller: 'CalculatorEnquiryCtrl'
      }
    },
    params: {
      details: null
    }
  });

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/app/home');
});
