angular.module('starter.controllers', ['starter.controllers.resorts', 'starter.controllers.holidays', 'starter.controllers.postcards', 'starter.controllers.articles', 'starter.controllers.availability', 'starter.controllers.calculator', 'starter.controllers.profile', 'starter.controllers.notifications'])

.controller('AppCtrl', function($rootScope, $scope, $timeout, $state, $auth, $http, $ionicPopup, $localStorage, $ionicPlatform, $cordovaDevice, $cordovaPushV5, $q, API, PROFILE_EVENTS, AUTH_EVENTS, LoginModalService, AuthPayloadFormatter, UnreadNotificationService, Device, DeviceService, Profile, PushNotificationPermission) {
  // With the new view caching in Ionic, Controllers are only called
  // when they are recreated or on app start, instead of every page change.
  // To listen for when this page is active (for example, to refresh data),
  // listen for the $ionicView.enter event:
  //$scope.$on('$ionicView.enter', function(e) {
  //});

  $scope.$storage = $localStorage;
  $scope.device = DeviceService.get();

  $scope.currentUser = null;
  $scope.setCurrentUser = function(user) {
    $scope.currentUser = user;
  }

  $scope.unread = UnreadNotificationService;
  UnreadNotificationService.update($scope.device);

  $scope.setCurrentUserProfile = function(profile) {
    if($scope.currentUser) {
      $scope.currentUser.email = profile.email;
      $scope.currentUser.owner_number = profile.owner_number;
      $scope.currentUser.favouriteResorts = {};
      angular.forEach(profile.favourite_resorts || [], function(id) {
        $scope.currentUser.favouriteResorts[id] = true;
      });
    }
  }

  // Open the login modal
  $scope.login = function() {
    LoginModalService.show();
  };

  var unbindLogin = $rootScope.$on(AUTH_EVENTS.login, function(e, user) {
    $scope.setCurrentUser(user);
    // Load profile details
    Profile.get().$promise.then(function(profile) {
      $rootScope.$broadcast(PROFILE_EVENTS.update, profile);
    });

    UnreadNotificationService.update($scope.device);
  });

  var unbindLogout = $rootScope.$on(AUTH_EVENTS.logout, function() {
    // On logout let the server know to remove association between device and user
    if($scope.currentUser) {
      $http({
        method: 'POST',
        url: API.url('logout'),
        data: {
          member_id: $scope.currentUser.member_id,
          platform: $scope.device.platform,
          uuid: $scope.device.uuid
        }
      });
    }
    $scope.setCurrentUser(null);
    UnreadNotificationService.update($scope.device);
  });

  var unbindProfile = $rootScope.$on(PROFILE_EVENTS.update, function(e, profile) {
    $scope.setCurrentUserProfile(profile);
  });

  // Login the user if the token has been saved locally
  if($auth.getToken()) {
    $rootScope.$broadcast(AUTH_EVENTS.login, AuthPayloadFormatter.format($auth.getPayload()))
  }

  // Check if push token already setup
  if($scope.$storage.pushToken) {
    // Call straight away, this initializes the handler
    PushNotificationPermission.request();
  } else {
    // Ask for push notifications after view becomes visible
    var viewLoadUnbind = $rootScope.$on('$ionicView.enter', function() {
      viewLoadUnbind();

      $timeout(3000).then(function() {
        // Only ask for push notifications once
        if(!$scope.$storage.hasRequestedPushNotifications) {
          PushNotificationPermission.request();
          $scope.$storage.hasRequestedPushNotifications = true;
        }
      })
    })
  }

  $scope.$on('$destroy', unbindLogin);
  $scope.$on('$destroy', unbindLogout);
  $scope.$on('$destroy', unbindProfile);
})

.controller('HomeCtrl', function($scope, $rootScope, $ionicSlideBoxDelegate, $q, $timeout, $ionicPlatform, HOLIDAY_EVENTS, AUTH_EVENTS, Holiday) {
  $scope.loaded = false;
  $scope.active = false;
  $scope.slideOptions = {
    // effect: 'coverflow'
  }

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;

    var promise = $scope.currentUser? Holiday.query({show_on_home: true, future: true}).$promise : $q.when([]);
    return promise
    .then(function(holidays) {
      $scope.loaded = true;
      var results = [];
      angular.forEach(holidays, function(holiday) {
        if(holiday.start_date) {
          var date = moment(holiday.start_date, 'YYYY-MM-DD HH:mm:ss');
          holiday.countdown = date.isValid()? date.fromNow() : null;
        } else {
          holiday.countdown = null;
        }
        return holiday;
      });

      $scope.holidays = holidays;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading holidays';
    })
    .finally(function() {
      $ionicSlideBoxDelegate.update()
    })
  }

  // Trigger reload
  var setUnloaded = function(e) {
    $scope.loaded = false;
    if($scope.active) $scope.loadResults();
  }
  var unbindCreate = $rootScope.$on(HOLIDAY_EVENTS.create, setUnloaded);
  var unbindEdit = $rootScope.$on(HOLIDAY_EVENTS.edit, setUnloaded);
  var unbindDelete = $rootScope.$on(HOLIDAY_EVENTS.delete, setUnloaded);
  var unbindLogin = $rootScope.$on(AUTH_EVENTS.login, setUnloaded);
  var unbindLogout = $rootScope.$on(AUTH_EVENTS.logout, setUnloaded);

  $scope.$on('$ionicView.enter', function() {
    $scope.active = true;
    if(!$scope.loaded || $scope.errorMessage || !$scope.holidays.length) {
      $scope.loadResults()
    }
  });

   $scope.$on('$ionicView.leave', function() {
    $scope.active = false;
   });

  $scope.$on('$destroy', unbindCreate);
  $scope.$on('$destroy', unbindEdit);
  $scope.$on('$destroy', unbindDelete);
  $scope.$on('$destroy', unbindLogin);
  $scope.$on('$destroy', unbindLogout);
})

.controller('ExploreNoPlanCtrl', function($scope, $state, $filter, Resort) {
  $scope.filters = {};

  $scope.submit = function() {
    $state.go('app.explore.no-plan.results', $scope.filters);
  }
})

.controller('ExploreNoPlanResultsCtrl', function($scope, $controller, $stateParams, Resort) {
  $scope.title = 'Recommended Resorts';

  $scope.getResultsPromise = function() {
    return Resort.recommend($stateParams).$promise;
  }

  // Initialize the super class and extend it.
  angular.extend(this, $controller('AbstractResortListingCtrl', {$scope: $scope}));
})

.controller('ExploreExperienceCtrl', function($scope, $state, Facility) {
  $scope.loaded = false;
  $scope.selection = [];

  $scope.facilityToggled = function(facility) {
    var idx = $scope.selection.indexOf(facility.record_id);
    if(idx > -1) {
      $scope.selection.splice(idx, 1);
    } else {
      $scope.selection.push(facility.record_id);
    }
  }

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.facilities = null;

    return Facility.query().$promise
    .then(function(facilities) {
      $scope.loaded = true;
      $scope.facilities = facilities;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading facilities';
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.continue = function() {
    $state.go('app.explore.experience.results', {
      facilities: $scope.selection.join(',')
    });
  }

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults(false)
    }
  });
})

.controller('ExploreExperienceResultsCtrl', function($scope, $controller, $stateParams, Resort) {
  $scope.title = 'Matching Resorts';

  $scope.getResultsPromise = function() {
    return Resort.query($stateParams).$promise;
  }

  // Initialize the super class and extend it.
  angular.extend(this, $controller('AbstractResortListingCtrl', {$scope: $scope}));
})

.controller('ResortSelectionModalCtrl', function($scope, Resort) {
  $scope.loaded = false;
  $scope.multiple = ($scope.multiple !== undefined)? $scope.multiple : true;
  // Map of resort ids to booleans indicating selection
  $scope.selection = [];

  $scope.$parent.$on('modal.shown', function() {
    if($scope.$parent.selection) $scope.selection = $scope.$parent.selection;
  });

  $scope.close = function() {
    $scope.$emit('resortselectionmodal.close');
  }

  // Toggling multiple resorts
  $scope.resortToggled = function(resort) {
    if(!$scope.multiple) {
      $scope.$emit('resortselectionmodal.complete', resort);
    }

    var idx = $scope.selection.indexOf(resort.record_id);
    if(idx > -1) {
      $scope.selection.splice(idx, 1);
    } else {
      $scope.selection.push(resort.record_id);
    }
  }

  // Submitting selection of multiple resorts
  $scope.continueClicked = function() {
    $scope.$emit('resortselectionmodal.complete', $scope.selection);
  }

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.countries = null;

    return Resort.query().$promise
    .then(function(resorts) {
      var countries = {};
      angular.forEach(resorts, function(resort) {
        if(!countries[resort.country]) {
          countries[resort.country] = {
            title: resort.country,
            states: {}
          };
        }
        if(!countries[resort.country].states[resort.state]) {
          countries[resort.country].states[resort.state] = {
            title: resort.state,
            resorts: []
          }
        }
        countries[resort.country].states[resort.state].resorts.push(resort);
      });
      $scope.countries = countries;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading resorts';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  if(!$scope.loaded) {
    $scope.loadResults();
  }
})

.controller('PlanningCtrl', function($scope, Config) {
  $scope.grabItUrl = Config.urls.grabIt;
  $scope.paymentUrl = Config.urls.payment;

  $scope.openSystemUrl = function(url) {
    window.open(url, '_system');
  }
})

.controller('PageCtrl', function($scope, $stateParams, Page) {
  $scope.slug = $scope.slug || $stateParams.slug;
  $scope.page = {
    page_name: 'Loading...',
    html: ''
  };

  // Load only once
  var unset = $scope.$on('$ionicView.enter', function() {
    $scope.loadResults();
    unset();
  });

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.$broadcast('loading.show');

    $scope.pagePromise = Page.get({slug: $scope.slug}).$promise
    .then(function(page) {
      if(!page || !page.page_id) {
        throw 'Missing page';
      }
      $scope.page = page;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading Page';
      $scope.page = {
        page_name: '404',
        html: '<div class="text-center">This page doesn\'t exist or cannot be accessed at this time. Please try again later</div>'
      };
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    });
  }
})

.controller('ActivityModalCtrl', function($scope, $ionicPopover, $ionicModal, $ionicSlideBoxDelegate, $auth, $q, LoginModalService) {
  $scope.hideAddButton = $scope.hideAddButton || false;
  var popoverScope = $scope.$new(true);
  $ionicPopover.fromTemplateUrl('templates/holidays/popover.html', {
    scope: popoverScope
  }).then(function(popover) {
    $scope.popover = popover;
  })

  $scope.openPopover = function($event) {
    var promise = $auth.getToken()? $q.when(true) : LoginModalService.show(true);

    promise.then(function() {
      popoverScope.activity = $scope.activity;
      $scope.popover.show($event);
    });
  }

  popoverScope.close = $scope.closePopover = function() {
    $scope.popover.hide();
  }

  popoverScope.success = function() {
    $scope.activityAdded = true;
  }

  // Create the gallery modal that we will use later
  $ionicModal.fromTemplateUrl('templates/gallery-modal.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.galleryModal = modal;
  });

  $scope.openGallery = function() {
    $scope.galleryImages = [];
    angular.forEach($scope.activity.images || [], function(image) {
      $scope.galleryImages.push({
        url: image.url_main
      });
    });

    $ionicSlideBoxDelegate.slide(0);
    $scope.galleryModal.show();
  }

  $scope.closeGallery = function() {
    $scope.galleryModal.hide();
  }

  //Cleanup the modals when we're done with them!
  $scope.$on('$destroy', function() {
    $scope.popover.remove();
    delete popoverScope;

    $scope.galleryModal.remove();
  })
})

.controller('ItineraryAddToHolidayPopoverCtrl', function($scope, $rootScope, $timeout, $ionicPopup, HOLIDAY_EVENTS, AUTH_EVENTS, Holiday) {
  $scope.loaded = false;

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.holidays = null;

    return Holiday.query().$promise
    .then(function(holidays) {
      angular.forEach(holidays, function(holiday) {
        if(holiday.start_date) {
          var date = moment(holiday.start_date, 'YYYY-MM-DD HH:mm:ss');
          holiday.countdown = date.isValid()? date.fromNow() : null;
        } else {
          holiday.countdown = null;
        }
        return holiday;
      });

      $scope.holidays = holidays;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading holidays';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  var setUnloaded = function(e) { $scope.loaded = false; }
  var unbindLogin = $rootScope.$on(AUTH_EVENTS.login, setUnloaded);
  $scope.$on('$destroy', unbindLogin);

  // Modal shown event not firing
  // User requested logout not clearing satellizer?
  $scope.$parent.$on('popover.shown', function() {
    if(!$scope.loaded || $scope.errorMessage || !$scope.holidays.length) {
      $scope.loadResults()
    }
  });

  $scope.addToHoliday = function(holiday) {
    $scope.showSuccess = false;
    Holiday.addActivity({holidayId: holiday.record_id, activityIds: $scope.activity.listing_id}, {}).$promise.then(function() {
      $rootScope.$broadcast(HOLIDAY_EVENTS.edit, holiday.record_id);

      $scope.showSuccess = true;
      $timeout(1000).then(function() {
        $scope.showSuccess = false;
        if($scope.success) {
          $scope.success();
        }
        $scope.close();
      })
    }, function() {
      $ionicPopup.alert({
        title: 'Error updating holiday',
        template: 'An error occurred. Please try again later',
      }).then(function() {
        $scope.close();
      });
    })
  }

  $scope.loadResults();
})

.controller('LoginModalCtrl', function($scope, $rootScope, $timeout, $auth, $ionicLoading, $ionicPopup, AUTH_EVENTS, AuthPayloadFormatter, DeviceService) {
  $scope.forms = {};
  $scope.loginData = {};
  $scope.registerData = {};
  $scope.action = 'login';
  $scope.device = DeviceService.get();

  // Triggered in the login modal to close it
  $scope.close = function() {
    $scope.loginData = {};
    $scope.registerData = {};
    if($scope.deferred) {
      $scope.deferred.reject('closed');
    }
    $scope.modal.hide();
  }

  $scope.login = function() {
    // Allow validation errors to show
    $scope.forms.login.$setSubmitted();

    if($scope.forms.login.$valid) {
      var loginData = angular.extend({
        platform: $scope.device.platform,
        uuid: $scope.device.uuid
      }, $scope.loginData);

      $ionicLoading.show();

      $auth.login(loginData).then(function(data) {
        $scope.loginData = {};
        $scope.forms.login.$setPristine();
        $scope.modal.hide();

        $scope.deferred.resolve({
          type: 'logged_in',
          profile_saved: data.data && data.data.profile_saved
        });

        $rootScope.$broadcast(AUTH_EVENTS.login, AuthPayloadFormatter.format($auth.getPayload()))
      })
      .catch(function() {
        $ionicPopup.alert({
          title: 'Incorrect Details',
          template: 'These details seem to be incorrect. Please try resetting your password or try again later.',
        })
      })
      .finally(function() {
        $scope.disabled = false;
        $ionicLoading.hide();
      })
    }
  }

  $scope.register = function() {
    // Allow validation errors to show
    $scope.forms.register.$setSubmitted();

    if($scope.forms.register.$valid) {
      var registerData = angular.extend({
        platform: $scope.device.platform,
        uuid: $scope.device.uuid
      }, $scope.registerData);

      $ionicLoading.show();

      $auth.signup(registerData).then(function(response) {
        $scope.registerData = {};
        $scope.forms.register.$setPristine();
        $scope.modal.hide();

        $auth.setToken(response.data.token);
        $rootScope.$broadcast(AUTH_EVENTS.login, AuthPayloadFormatter.format($auth.getPayload()))
        $scope.deferred.resolve({
          type: 'registered',
          profile_saved: false
        });
      })
      .catch(function() {
        $ionicPopup.alert({
          title: 'Incorrect Details',
          template: 'These details seem to be incorrect. You might already have an account. Please try again later.',
        })
      })
      .finally(function() {
        $scope.disabled = false;
        $ionicLoading.hide();
      })
    }
  }
})

.controller('LocationSelectionModalCtrl', function($scope, $cordovaGeolocation, $timeout) {
  $scope.selection = null;
  $scope.location = null;

  $scope.map = {
    center: {
      latitude: -30.265086,
      longitude: 153.13719
    },
    clickedMarker: {
      id: 0,
      options:{
      }
    },
    bounds: {},
    zoom: 3,
    control: {},
    events: {
      click: function(mapModel, eventName, originalEventArgs) {
        $scope.location = null;
        var e = originalEventArgs[0];
        var lat = e.latLng.lat(), lon = e.latLng.lng();
        $scope.$broadcast('locationselectionmodal.select', {latitude: lat, longitude: lon, name: lat + ', ' + lon});
        // Scope apply required because this event handler is outside of the angular domain
        $scope.$evalAsync();
      }
    }
  };

  $scope.$on('locationselectionmodal.select', function(e, selection) {
    $scope.selection = selection;
    if(selection) {
      var lat = selection.latitude, lon = selection.longitude;
      $scope.map.center.latitude = lat;
      $scope.map.center.longitude = lon;
      $scope.map.clickedMarker = {
        id: 0,
        latitude: lat,
        longitude: lon
      };
    } else {
      $scope.map.clickedMarker = null;
    }
  });

  $scope.$parent.$on('modal.shown', function() {
    if($scope.map.control.refresh) {
      $scope.map.control.refresh($scope.map.center);
    }
  })

  $scope.$watch('location', function(newValue, oldValue) {
    if(newValue && newValue.geometry) {
      var lat = newValue.geometry.location.lat(), lon = newValue.geometry.location.lng();
      $scope.$broadcast('locationselectionmodal.select', {latitude: lat, longitude: lon, name: newValue.formatted_address});
    }
  });

  $scope.close = function() {
    $scope.$emit('locationselectionmodal.close');
  }

  $scope.continueClicked = function() {
    $scope.map.clickedMarker = null;
    $scope.$emit('locationselectionmodal.complete', $scope.selection);
  }
})

.controller('ItineraryTemplateModalCtrl', function($scope, $rootScope, $ionicModal, $ionicPopup, $ionicLoading, HOLIDAY_EVENTS, Holiday, ItineraryTemplate) {
  $scope.loaded = false;

  $scope.loadResults = function(pulledToRefresh) {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.templates = null;

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    ItineraryTemplate.getBySiteId($scope.holiday.resort.getawaysp_site_id).success(function(templates) {
      $scope.templates = templates;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading templates';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  var checkReloadRequired = function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  }
  $scope.$parent.$on('modal.shown', checkReloadRequired);

  var addActivityIds = function(ids) {
    $ionicLoading.show();
    return Holiday.addActivity({holidayId: $scope.holiday.record_id, activityIds: ids.join(',')}, {}).$promise.then(function() {
      $rootScope.$broadcast(HOLIDAY_EVENTS.edit, $scope.holiday.record_id);
      $ionicPopup.alert({
        title: 'Success!',
        template: 'Item added to Holiday Itinerary',
      })
    }, function() {
      $ionicPopup.alert({
        title: 'Error updating holiday',
        template: 'An error occurred. Please try again later',
      })
    }).finally(function() {
      $ionicLoading.hide();
    })
  }

  $scope.addAllActivities = function(listings) {
    var ids = [];
    angular.forEach(listings, function(activity) {
      ids.push(activity.listing_id);
    })
    return addActivityIds(ids);
  }

  $scope.addActivity = function(e, activity) {
    e.stopPropagation()
    return addActivityIds([activity.listing_id]);
  }

  var activityScope = $scope.$new(true);
  activityScope.hideAddButton = true;
  // Create the activity modal that we will use later
  $ionicModal.fromTemplateUrl('templates/activity-modal.html', {
    scope: activityScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  $scope.openModal = function(activity) {
    activityScope.activity = activity;
    $scope.modal.show();
  }

  activityScope.close = function() {
    $scope.modal.hide();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete activityScope;
  });
})