angular.module('starter.controllers.availability', [])

.controller('AvailabilityCtrl', function($scope, $ionicModal, $filter, $state) {
  $scope.filters = {};
  $scope.resortSelection = {};

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.modal.hide();
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resorts) {
    $scope.filters.resorts = resorts;
    $scope.openResults();
  });

  $scope.openModal = function() {
    $scope.modal.show();
  }

  $scope.openResults = function() {
    var params = {
      resorts: ($scope.filters.resorts || []).join(','),
      from_date: $filter('date')($scope.filters.from_date, 'yyyy-MM-dd'),
      to_date: $filter('date')($scope.filters.to_date, 'yyyy-MM-dd')
    };

    $scope.modal.hide();
    $state.go('app.availability.results', params);
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete resortSelectionScope;
  });
})

.controller('AvailabilityResultsCtrl', function($scope, $stateParams, $ionicScrollDelegate, $http, Config, API) {
  $scope.bookingUrl = Config.urls.booking;
  var sidebar = angular.element(document.getElementById('availability-result-sidebar'));
  var scrollDelegate = $ionicScrollDelegate.$getByHandle('availability-results');

  $scope.adjustSidebar = function() {
    var position = scrollDelegate.getScrollPosition();
    var transform = 'translate(0, ' + (-position.top) + 'px)';
    sidebar.css('-ms-transform', transform);
    sidebar.css('-webkit-transform', transform);
    sidebar.css('transform', transform);
  }

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.availability = null;
    $scope.loading = true;

    $http.get(API.url('availability/search'), {
      params: $stateParams
    })
    .success(function(availability) {
      var availabilityData = null;
      var resorts = null;

      angular.forEach(availability, function(record) {
        var hashKey = record.date_from + '-' + record.date_to;
        //Setup data
        availabilityData = availabilityData || {};
        resorts = resorts || {};

        if(!availabilityData[hashKey]) {
            availabilityData[hashKey] = {
                date_from: record.date_from,
                date_to: record.date_to,
                week_name: record.week_name,
                availability: {}
            };
        }

        resorts[record.resort_name] = record.resort_id;
        availabilityData[hashKey].availability[record.resort_id] = record.availability;
      });

      $scope.availabilityData = availabilityData;
      $scope.resorts = resorts;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading results';
    })
    .finally(function() {
      $scope.loading = false;

      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.loadResults();
})