angular.module('starter.controllers.calculator', [])

.controller('CalculatorCtrl', function($scope, $ionicModal, $filter, $state, $stateParams, Resort) {
  $scope.forms = {};
  $scope.filters = {};
  $scope.resortSelection = {};
  $scope.roomTypes = [];

  $scope.loadRoomTypes = function() {
    $scope.roomTypeLoading = true;
    $scope.errorMessage = null;
    $scope.filters.room_type = null;
    Resort.roomTypes({resortId: $scope.filters.resort_id}).$promise
    .then(function(roomTypes) {
      $scope.roomTypes = roomTypes;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading room types';
    })
    .finally(function() {
      $scope.roomTypeLoading = false;
    })
  }

  if($stateParams.resortId) {
    $scope.filters.resort_id = $stateParams.resortId;

    $scope.resortLoading = true;
    $scope.errorMessage = null;
    Resort.get({resortId: $stateParams.resortId}).$promise
    .then(function(resort) {
      $scope.resort = resort;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading resort';
    })
    .finally(function() {
      $scope.resortLoading = false;
    })
    $scope.loadRoomTypes();
  }

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  resortSelectionScope.multiple = false;
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.modal.hide();
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resort) {
    $scope.filters.resort_id = resort.record_id;
    $scope.resort = resort;
    $scope.modal.hide();
    $scope.loadRoomTypes();
  });

  $scope.openModal = function() {
    $scope.modal.show();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete resortSelectionScope;
  });

  $scope.submitForm = function() {
    // Allow validation errors to show
    $scope.forms.filters.$setSubmitted();

    if($scope.forms.filters.$valid) {
      var params = {
        resortId: $scope.filters.resort_id,
        room_type: $scope.filters.room_type,
        start_date: $filter('date')($scope.filters.start_date, 'yyyy-MM-dd'),
        end_date: $filter('date')($scope.filters.end_date, 'yyyy-MM-dd')
      };

      $state.go('app.calculator.results', params);
    }
  }
})

.controller('CalculatorResultsCtrl', function($scope, $state, $stateParams, $ionicPopover, Resort) {
  $scope.credits = [];
  $scope.loaded = false;

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.$broadcast('loading.show');

    return Resort.roomCredits({
      resortId: $stateParams.resortId,
      room_type: $stateParams.room_type,
      start_date: $stateParams.start_date,
      end_date: $stateParams.end_date
    })
    .$promise
    .then(function(data) {
      $scope.loaded = true;
      $scope.credits = data.credits;
      $scope.total = data.total;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading results';
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    })
  }

  $ionicPopover.fromTemplateUrl('templates/calculator/comparison-popover.html', {
    scope: angular.extend($scope.$new(), {filters: {
      resort_id: $stateParams.resortId,
      room_type: $stateParams.room_type,
      start_date: $stateParams.start_date,
      end_date: $stateParams.end_date
    }})
  }).then(function(popover) {
    $scope.popover = popover;
  })

  $scope.openPopover = function($event) {
    $scope.popover.show($event);
  };

  $scope.$on('$destroy', function() {
    $scope.popover.remove();
  });

  $scope.enquiryClick = function() {
    var details = angular.extend({total: $scope.total}, $stateParams);
    details.resort_id = details.resortId;
    delete details.resortId;
    $state.go('app.calculator.enquiry', {resortId: $stateParams.resortId, details: details});
  }

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  });
})

.controller('CalculatorComparisonPopoverCtrl', function($scope, $state, $ionicHistory, Resort) {
  $scope.filters = $scope.filters || {};
  $scope.loaded = false;

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.loaded = false;
    $scope.results = null;
    $scope.$broadcast('loading.show');

    return Resort.roomCreditsCompare({
      resortId: $scope.filters.resort_id,
      start_date: $scope.filters.start_date,
      end_date: $scope.filters.end_date
    }).$promise
    .then(function(results) {
      $scope.results = results;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading results';
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.switchRoomType = function(roomType) {
    if($scope.filters.room_type != roomType) {
      $scope.popover.hide();
      $state.go($state.current, {
        resortId: $scope.filters.resort_id,
        room_type: roomType,
        start_date: $scope.filters.start_date,
        end_date: $scope.filters.end_date
      });
      $ionicHistory.nextViewOptions({
        disableAnimate: true,
      });
    }
  }

  $scope.$on('popover.shown', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults();
    }
  })
})

.controller('CalculatorEnquiryCtrl', function($scope, $state, $stateParams, $ionicHistory, $ionicPopup, $http, API, Resort) {
  $scope.details = $stateParams.details || {};
  $scope.enquiry = {};
  $scope.forms = {};

  $scope.resortLoading = true;
  $scope.errorMessage = null;

  if($scope.details.resort_id) {
    Resort.get({resortId: $scope.details.resort_id}).$promise
    .then(function(resort) {
      $scope.resort = resort;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading resort';
    })
    .finally(function() {
      $scope.resortLoading = false;
    })
  } else {
    $scope.errorMessage = 'Error loading details';
    $scope.resortLoading = false;
  }

  // Prefill email address if available
  $scope.$on('$ionicView.enter', function() {
    if($scope.currentUser && $scope.currentUser.email && !$scope.enquiry.email) {
      $scope.enquiry.email = $scope.currentUser.email;
    }
  });

  $scope.submitForm = function() {
    // Allow validation errors to show
    $scope.forms.enquiry.$setSubmitted();

    if($scope.forms.enquiry.$valid) {
      if($scope.sending) {
        return;
      }
      $scope.sending = true;

      return $http.post(API.url('calculator/enquire'), angular.extend({details: $scope.details}, $scope.enquiry))
      .then(function(result) {
        $ionicPopup.alert({
          title: 'Email Sent',
          template: 'Thanks for your enquiry. We will respond as soon as possible.',
        }).then(function() {
          $scope.enquiry = {};
          $scope.forms.enquiry.$setPristine();

          if($ionicHistory.backView()) {
            $ionicHistory.goBack();
          }
        })
      })
      .catch(function(message) {
        $ionicPopup.alert({
          title: 'Error',
          template: 'Unable to send your enquiry. Please try again later.',
        })
      })
      .finally(function() {
        $scope.sending = false;
      });
    }
  }
})