angular.module('starter.controllers.holidays', [])

.controller('HolidayListingCtrl', function($scope, $rootScope, $state, HOLIDAY_EVENTS, LoginModalService, Holiday) {
  $scope.loaded = false;

  $scope.loadResults = function(pulledToRefresh) {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.holidays = null;

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Holiday.query().$promise
    .then(function(holidays) {
      $scope.loaded = true;
      angular.forEach(holidays, function(holiday) {
        if(holiday.start_date) {
          var date = moment(holiday.start_date, 'YYYY-MM-DD HH:mm:ss');
          holiday.countdown = date.isValid()? date.fromNow() : null;
        } else {
          holiday.countdown = null;
        }
      });

      $scope.holidays = holidays;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading holidays';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  // Trigger reload
  var setUnloaded = function(e) { $scope.loaded = false; }

  $scope.openNotifications = function(e, holiday) {
    e.stopPropagation();
    e.preventDefault();
    $state.go('app.notifications', {resortId: holiday.resort.record_id});
  }

  var unbindCreate = $rootScope.$on(HOLIDAY_EVENTS.create, setUnloaded);
  var unbindEdit = $rootScope.$on(HOLIDAY_EVENTS.edit, setUnloaded);
  var unbindDelete = $rootScope.$on(HOLIDAY_EVENTS.delete, setUnloaded);

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults(false)
    }
  });

  $scope.$on('$destroy', unbindCreate);
  $scope.$on('$destroy', unbindEdit);
  $scope.$on('$destroy', unbindDelete);
})

.controller('HolidayCreateCtrl', function($scope, $rootScope, $state, $stateParams, $ionicHistory, $ionicPopup, $ionicModal, $ionicLoading, HOLIDAY_EVENTS, Resort, Holiday) {
  $scope.forms = {};

  $scope.reset = function() {
    var today = new Date();
    var tomorrow = new Date();
    tomorrow.setDate(today.getDate() + 1);

    $scope.holiday = {
      start_date: today,
      end_date: tomorrow,
      show_on_home: true
    };
  }

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  resortSelectionScope.multiple = false;
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.modal.hide();
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resort) {
    $scope.holiday.resort_id = resort.record_id;
    $scope.holiday.resort = resort;
    $scope.modal.hide();
  });

  $scope.openModal = function() {
    $scope.modal.show();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete resortSelectionScope;
  });

  $scope.submitForm = function() {
    // Allow validation errors to show
    $scope.forms.holiday.$setSubmitted();

    if($scope.forms.holiday.$valid) {
      $ionicLoading.show();

      new Holiday($scope.holiday).$create()
      .then(function() {
        $scope.holiday = null;
        $scope.forms.holiday.$setPristine();
        $scope.reset();

        $rootScope.$broadcast(HOLIDAY_EVENTS.create);

        $ionicHistory.nextViewOptions({
          disableBack: true
        });
        $state.go('app.holidays');
      })
      .catch(function(response) {
        // Validation error
        if(response.status == 422) {
          $ionicPopup.alert({
            title:'Validation Error',
            scope: angular.extend($scope.$new(), { validationErrors: response.data }),
            templateUrl: 'templates/popup-validation-errors.html'
          })
        } else {
          $scope.errorMessage = 'Error creating holiday';
        }
      })
      .finally(function() {
        $ionicLoading.hide();
      })
    }
  }

  $scope.reset();
})

.controller('HolidayShowCtrl', function($scope, $rootScope, $state, $stateParams, $ionicActionSheet, $ionicPopup, $ionicHistory, $ionicModal, HOLIDAY_EVENTS, Resort, Holiday, Activity, ItineraryTemplate) {
  $scope.loaded = false;
  $scope.editing = false;
  $scope.title = 'Loading...';
  $scope.today = new Date();

  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.shouldShowDelete = false;
    $scope.$broadcast('loading.show');
    $scope.today = new Date();

    return Holiday.get({holidayId: $stateParams.holidayId}).$promise
    .then(function(holiday) {
      $scope.holiday = holiday;

      $scope.holiday.start_date = moment(holiday.start_date, 'YYYY-MM-DD HH:mm:ss').toDate();
      $scope.holiday.end_date = moment(holiday.end_date, 'YYYY-MM-DD HH:mm:ss').toDate();

      $scope.title = holiday.name;
      $scope.loaded = true;
    })
    .then($scope.loadActivities)
    .catch(function() {
      $scope.title = 'Error';
      $scope.errorMessage = 'Error loading holiday. Please try again';
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.toggleEditing = function() {
    $scope.editing = !$scope.editing;
  }

  $scope.removeActivity = function(activity, index) {
    if($scope.activities) {
      // Remove the item from the activities array
      var item = $scope.activities.splice(index, 1);

      Holiday.removeActivity({holidayId: $scope.holiday.record_id, activityId: activity.listing_id}, {}).$promise.then(function() {
        // Success
        var previousLoadedState = $scope.loaded;
        $rootScope.$broadcast(HOLIDAY_EVENTS.edit, $scope.holiday.record_id);

        // Restore the loaded state for this particular view (we know the ui will be up to date after firing this particular event)
        $scope.loaded = previousLoadedState;
      }, function() {
        $ionicPopup.alert({
          title: 'Error removing activity',
          template: 'An error occurred. Please try again later',
        }).then(function() {
          // Add the item back into the array
          $scope.activities.splice(index, 0, item[0]);
        });
      })
    }
  }

  $scope.loadActivities = function() {
    return Activity.getByIds($scope.holiday.activities).then(function(activities) {
      $scope.activities = activities.data;
    })
  }

  $scope.deleteHoliday = function() {
    // Show the action sheet
    var hideSheet = $ionicActionSheet.show({
      cancelText: 'Cancel',
      cancel: function() {
        hideSheet();
      },
      destructiveText: 'Delete Holiday',
      destructiveButtonClicked: function() {
        $scope.holiday.$delete().then(function() {
          $rootScope.$broadcast(HOLIDAY_EVENTS.delete, $stateParams.holidayId);
          hideSheet();

          if($ionicHistory.backView()) {
            $ionicHistory.goBack();
          } else {
            $ionicHistory.nextViewOptions({
              disableBack: true
            });
            $state.go('app.holidays', {}, {
              reload: true
            });
          }
        }, function() {
          $ionicPopup.alert({
            title: 'Error deleting holiday',
            template: 'An error occurred. Please try again later',
          }).then(function() {
            hideSheet();
          });
        })
      }
    });
  }

  var activityScope = $scope.$new(true);
  activityScope.hideAddButton = true;
  // Create the activity modal that we will use later
  $ionicModal.fromTemplateUrl('templates/activity-modal.html', {
    scope: activityScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  $scope.openModal = function(activity) {
    activityScope.activity = activity;
    $scope.modal.show();
  }

  activityScope.close = function() {
    $scope.modal.hide();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete activityScope;
  });

  var itineraryTemplateScope = $scope.$new(true);
  // Create the itineraryTemplate modal that we will use later
  $ionicModal.fromTemplateUrl('templates/itinerary-template-modal.html', {
    scope: itineraryTemplateScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.itineraryTemplateModal = modal;
  });

  $scope.openItineraryTemplateModal = function() {
    itineraryTemplateScope.holiday = $scope.holiday;
    $scope.itineraryTemplateModal.show();
  }

  itineraryTemplateScope.close = function() {
    $scope.itineraryTemplateModal.hide();
    checkReloadRequired();
  }

  $scope.$on('$destroy', function() {
    $scope.itineraryTemplateModal.remove();
    delete itineraryTemplateScope;
  });

  // Trigger reload
  var unbindEdit = $rootScope.$on(HOLIDAY_EVENTS.edit, function(e, id) {
    if(id == $stateParams.holidayId) {
      $scope.loaded = false;
    }
  })

  var checkReloadRequired = function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  }
  $scope.$on('$ionicView.enter', checkReloadRequired);

  $scope.$on('$destroy', unbindEdit);
})

.controller('HolidayEditCtrl', function($scope, $rootScope, $state, $stateParams, $ionicHistory, $ionicPopup, $ionicLoading, $ionicModal, HOLIDAY_EVENTS, Resort, Holiday) {
  $scope.loaded = false;
  $scope.title = 'Loading...';
  $scope.forms = {};


  $scope.loadResults = function() {
    $scope.loaded = false;
    $scope.errorMessage = null;
    return Holiday.get({holidayId: $stateParams.holidayId}).$promise
    .then(function(holiday) {
      $scope.holiday = holiday;

      $scope.holiday.start_date = moment(holiday.start_date, 'YYYY-MM-DD HH:mm:ss').toDate();
      $scope.holiday.end_date = moment(holiday.end_date, 'YYYY-MM-DD HH:mm:ss').toDate();

      $scope.title = 'Edit Holiday';
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading holiday. Please try again';
    })
    .finally(function() {
      $scope.loaded = true;
      $scope.$broadcast('loading.hide');
    });
  }

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  resortSelectionScope.multiple = false;
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.modal.hide();
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resort) {
    $scope.holiday.resort_id = resort.record_id;
    $scope.holiday.resort = resort;
    $scope.modal.hide();
  });

  $scope.openModal = function() {
    $scope.modal.show();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete resortSelectionScope;
  });

  $scope.submitForm = function() {
    // Allow validation errors to show
    $scope.forms.holiday.$setSubmitted();

    if($scope.forms.holiday.$valid) {
      $ionicLoading.show();

      $scope.holiday.$save()
      .then(function() {
        $rootScope.$broadcast(HOLIDAY_EVENTS.edit, $stateParams.holidayId);

        if($ionicHistory.backView()) {
          $ionicHistory.goBack();
        } else {
          $ionicHistory.nextViewOptions({
            disableBack: true
          });
          $state.go('app.holidays', {}, {
            reload: true
          });
        }
      })
      .catch(function(response) {
        // Validation error
        if(response.status == 422) {
          $ionicPopup.alert({
            title: 'Validation Error',
            scope: angular.extend($scope.$new(), { validationErrors: response.data }),
            templateUrl: 'templates/popup-validation-errors.html'
          })
        } else {
          $scope.errorMessage = 'Error updating holiday';
        }
      })
      .finally(function() {
        $ionicLoading.hide();
      })
    }
  }

  // Trigger reload
  var unbindEdit = $rootScope.$on(HOLIDAY_EVENTS.edit, function(e, id) {
    if(id == $stateParams.holidayId) {
      $scope.loaded = false;
    }
  });

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  });

  $scope.$on('$destroy', unbindEdit);
})

.controller('HolidayShareCtrl', function($scope, $http, $q, $timeout, $ionicPopup, $ionicLoading, $ionicPlatform, $cordovaCamera, $cordovaSocialSharing, $rootScope, HOLIDAY_EVENTS, Holiday, WorldMarkJourneyAPI, Profile, profilePromise, holidayPromise) {
  $scope.profile = profilePromise;
  $scope.holiday = holidayPromise;

  $scope.shareDetails = {
    story: ''
  };
  $scope.images = [];
  $scope.sharing = false;
  $scope.sharingCompleted = false;
  $scope.progress = 0;

  $scope.removeImage = function(index) {
    if($scope.images) {
      $scope.images.splice(index, 1);
    }
  }

  $scope.share = function() {
    $scope.sharing = true;
    $scope.sharingCompleted = false;
    $scope.errorMessage = null;
    $scope.$broadcast('loading.show');
    $scope.sharingStatus = 'Starting...';
    $scope.progress = 0;

    var holidayUploadProgress = 20;
    var imageUploadProgress = (100 - holidayUploadProgress) / $scope.images.length;

    var steps = [];
    var step = 0;
    var holidayId = null;
    var kilometres = null;

    var shareDetails = {};

    steps.push(function() {
      $scope.sharingStatus = 'Creating Holiday...';
      return $http.post(WorldMarkJourneyAPI.url('holiday/'), {
        user_id: $scope.profile.worldmark_journey_id,
        key: $scope.profile.worldmark_journey_key,

        name: $scope.holiday.name,
        end_date: $scope.holiday.end_date,
        start_date: $scope.holiday.start_date,
        resort_id: $scope.holiday.resort.worldmark_journey_resort_id,
        resort_name: $scope.holiday.resort.name,
        resort_latitude: $scope.holiday.resort.location_latitude,
        resort_longitude: $scope.holiday.resort.location_longitude,
        story: $scope.shareDetails.story
      }).then(function(results) {
        holidayId = results.data.data.holiday_id;
        kilometres = results.data.data.kilometres;
        shareDetails.url = results.data.data.share_url;
        shareDetails.message = results.data.data.share_message;
        shareDetails.image = results.data.data.share_image;

        $scope.progress += holidayUploadProgress;
        step++;
        if(step < steps.length) {
          return steps[step]();
        }
      });
    })

    angular.forEach($scope.images, function(image, index) {
      steps.push(function() {
        $scope.sharingStatus = 'Uploading image #' + (index + 1) + '...';
        return $http.post(WorldMarkJourneyAPI.url('holiday/' + holidayId + '/image'), {
          user_id: $scope.profile.worldmark_journey_id,
          key: $scope.profile.worldmark_journey_key,

          image: image.src
        }).then(function(results) {
          $scope.progress += imageUploadProgress;
          step++;
          if(step < steps.length) {
            return steps[step]();
          }
        });
      })
    })

    return steps[step]().then(function() {
      $scope.progress = 100;
      $scope.shareDetails = {};
      $scope.images = [];
      $scope.sharingStatus = 'Completed';

      // Mark holiday as shared
      Holiday.shared({holidayId: $scope.holiday.record_id}, {});
      $rootScope.$broadcast(HOLIDAY_EVENTS.edit, $scope.holiday.record_id);

      return $ionicPopup.alert({
        title: 'Holiday Created!',
        template: kilometres + ' WorldMark Kilometres have been added to your WorldMark Journey!'
      }).then(function() {
        return $ionicPopup.confirm({
          title: 'Share',
          template: 'Would you like to share your holidays on facebook?',
          okText: 'Yes',
          cancelText: 'No'
        }).then(function(res) {
          if(res) {
            $cordovaSocialSharing.shareViaFacebook(shareDetails.message || '', shareDetails.image, shareDetails.url);
          }
        });
      });
    }).catch(function() {
      $scope.errorMessage = 'Error occurred...';
    }).finally(function() {
      $scope.sharingCompleted = true;
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.closeShare = function() {
    // Reset
    $scope.sharing = false;
  }

  $ionicPlatform.ready(function() {
    if(!window.navigator || !window.navigator.camera) {
      return;// Unsupported platform
    }

    var defaultOptions = {
      destinationType: navigator.camera.DestinationType.DATA_URL,
      targetWidth: 625,
      targetHeight: 625,
      quality: 94,
      // allowEdit: true,
      saveToPhotoAlbum: false,
      correctOrientation: true,
      encodingType: navigator.camera.EncodingType.JPEG,
    }

    var getCameraPopoverOptions = function(event) {
      return {
        x: event.pageX || 0,
        y: event.pageY || 0
      };
    }

    var capture = function(event, options) {
      if($scope.images.length >= 5) {
        return $ionicPopup.alert({
          title: 'Maximum images reached',
          template: 'Only 5 images can be uploaded via the app. To add more visit the My WorldMark Journey website'
        }).finally(function() {
          return $q.reject('maximum_images')
        })
      }
      return $q.when($ionicLoading.show()).then(function() {return $cordovaCamera.getPicture(options)
      }).then(function(imageData) {
        $scope.images.push({src: 'data:image/jpeg;base64,' + imageData})

      }, function() {
        // Error
        $ionicPopup.alert({
          title: 'Error choosing photo',
          template: 'An error occurred. Please try again later',
        });
      }).finally($ionicLoading.hide)
    }

    $scope.takePhoto = function(event) {
      var options = angular.extend({}, defaultOptions, {
        sourceType: navigator.camera.PictureSourceType.CAMERA,
        popoverOptions: getCameraPopoverOptions(event),
      });
      return capture(event, options);
    }

    $scope.choosePhoto = function(event) {
      var options = angular.extend({}, defaultOptions, {
        sourceType: navigator.camera.PictureSourceType.PHOTOLIBRARY,
        popoverOptions: getCameraPopoverOptions(event),
      });
      return capture(event, options);
    }
  });
})