angular.module('starter.controllers.notifications', [])

.controller('NotificationListingCtrl', function($scope, $rootScope, $state, $stateParams, $q, AUTH_EVENTS, UnreadNotificationService, Notification) {
  $scope.loaded = false;
  $scope.page = 0;
  $scope.hasMorePages = true;
  $scope.notifications = [];

  $scope.openNotification = function(notification) {
    if(!notification.read_at) {
      UnreadNotificationService.markRead($scope.device, notification);
    }
    if(notification.route) {
      var match = notification.route.match(/(.+)\((.*)\)$/);
      if(match) {
        try {
          var params = JSON.parse(match[2]);
          $state.go(match[1], params);
        } catch(e) {}
      } else {
        $state.go(notification.route);
      }
    }
  }

  $scope.reset = function() {
    $scope.page = 0;
    $scope.loaded = false;
    $scope.hasMorePages = true;
    $scope.notifications = [];
  }

  $scope.loadResults = function(reset, type) {
    $scope.loaded = false;
    $scope.errorMessage = null;

    if(reset) {
      $scope.reset();
    } else if(!$scope.hasMorePages) {
      // Nothing to do
      return $q.when();
    }

    if(type == 'refresher') {
    } else if(type == 'infinite-scroll') {
    } else {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Notification.query({
      platform: $scope.device.platform,
      uuid: $scope.device.uuid,
      page: $scope.page + 1,
      resort_id: $stateParams.resortId? $stateParams.resortId : null
    }).$promise
    .then(function(notifications) {
      $scope.loaded = true;
      $scope.page++;

      if(notifications.length) {
        angular.forEach(notifications, function(notification) {
          var date = moment(notification.created_at, 'YYYY-MM-DD HH:mm:ss');
          notification.age = date.isValid()? date.fromNow() : null;
          $scope.notifications.push(notification);
        });
      } else {
        $scope.hasMorePages = false;
      }
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading notifications';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
      $scope.$broadcast('scroll.infiniteScrollComplete');
    });
  }

  // Trigger reload
  var setUnloaded = function(e) { $scope.loaded = false; }
  var unbindLogin = $rootScope.$on(AUTH_EVENTS.login, setUnloaded);

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults(true, false)
    }
  });

  $scope.$on('$destroy', unbindLogin);
})