angular.module('starter.controllers.postcards', [])

.controller('PostcardSelectPhotoCtrl', function($scope, $q, $ionicLoading, $cordovaCamera, $ionicPopup, $ionicPlatform, PostcardData) {
  $ionicPlatform.ready(function() {
    if(!window.navigator || !window.navigator.camera) {
      return;// Unsupported platform
    }

    $scope.base64ImageUri = null;
    var defaultOptions = {
      destinationType: navigator.camera.DestinationType.DATA_URL,
      targetWidth: 625,
      targetHeight: 625,
      quality: 94,
      // allowEdit: true,
      saveToPhotoAlbum: false,
      correctOrientation: true,
      encodingType: navigator.camera.EncodingType.JPEG,
    }

    var getCameraPopoverOptions = function(event) {
      return {
        x: event.pageX || 0,
        y: event.pageY || 0
      };
    }

    var capture = function(event, options) {
      return $q.when($ionicLoading.show()).then(function() {return $cordovaCamera.getPicture(options)
      }).then(function(imageData) {
        $scope.base64ImageUri = 'data:image/jpeg;base64,' + imageData;
        PostcardData.merge({image: $scope.base64ImageUri});
      }, function() {
        // Error
        $ionicPopup.alert({
          title: 'Error choosing photo',
          template: 'An error occurred. Please try again later',
        });
      }).finally($ionicLoading.hide)
    }

    $scope.takePhoto = function(event) {
      var options = angular.extend({}, defaultOptions, {
        sourceType: navigator.camera.PictureSourceType.CAMERA,
        popoverOptions: getCameraPopoverOptions(event),
      });
      return capture(event, options);
    }

    $scope.choosePhoto = function(event) {
      var options = angular.extend({}, defaultOptions, {
        sourceType: navigator.camera.PictureSourceType.PHOTOLIBRARY,
        popoverOptions: getCameraPopoverOptions(event),
      });
      return capture(event, options);
    }
  });
})

.controller('PostcardChooseTemplateCtrl', function($scope, PostcardTemplate, PostcardData) {
  $scope.selected = null;

  $scope.select = function(template) {
    $scope.selected = template;
    PostcardData.merge({template: template.slug});
  }

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.templates = null;
    $scope.selected = null;

    return PostcardTemplate.query().$promise
    .then(function(templates) {
      $scope.templates = templates;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading templates';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.loadResults();
})

.controller('PostcardDetailsCtrl', function($scope, $state, PostcardData) {
  $scope.forms = {};
  $scope.postcard = {};

  $scope.next = function() {
    // Allow validation errors to show
    $scope.forms.postcard.$setSubmitted();

    if($scope.forms.postcard.$valid) {
      PostcardData.merge($scope.postcard);
      $state.go('app.postcard.output');
    } else {
      // No dice
    }
  }
})

.controller('PostcardOutputCtrl', function($scope, $cordovaSocialSharing, $ionicHistory, $ionicPopup, $ionicPlatform, $http, $state, $ionicPopup, PostcardData, API) {
  $scope.$on('$ionicView.enter', function(e) {
    $scope.loadResults(PostcardData.get());
  });

  $scope.loadResults = function(postcardData) {
    $scope.loaded = false;
    $scope.errorMessage = null;
    $scope.imageUriPrefix = 'data:image/jpeg;base64,';
    $scope.resultBase64Image = null;

    return $http.post(API.url('postcards/generate'), postcardData)
    .then(function(result) {
      $scope.resultBase64Image = result.data;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error generating postcard';
    })
    .finally(function() {
      $scope.$broadcast('loading.hide');
    });
  }

  $scope.returnHome = function() {
    $ionicHistory.nextViewOptions({
      historyRoot: true,
      disableBack: true,
    });
    $ionicHistory.clearCache();
    $state.go('app.home');
  }

  $scope.done = function() {
    if($scope.resultBase64Image && !$scope.shared && window.imageSaver) {
      $ionicPopup.confirm({
        title: 'Save',
        template: 'Would you like to save the postcard to your device?',
        okText: 'Yes',
        cancelText: 'No'
      }).then(function(res) {
        if(res) {
          window.imageSaver.saveBase64Image({
            data: $scope.resultBase64Image
          }, function() {
            $scope.returnHome();
          }, function() {
            $scope.errorMessage = 'Error saving postcard';
          });
        } else {
          $scope.returnHome();
        }
      });
    } else {
      $scope.returnHome();
    }
  }

  $scope.share = function() {
    if(!window.navigator || !window.navigator.camera) {
      return;// Unsupported platform
    }

    if($scope.isSharing || !$scope.resultBase64Image) {
      return;
    }

    $scope.isSharing = true;
    $cordovaSocialSharing
    .share(null, 'postcard-' + (new Date()).getTime() + '.jpg', $scope.imageUriPrefix + $scope.resultBase64Image, null)
    .then(function() {
      // Success
      $scope.shared = true;
    }, function() {
      // Error
      $ionicPopup.alert({
        title: 'Error sharing photo',
        template: 'An error occurred. Please try again later',
      });
    })
    .finally(function() {
      $scope.isSharing = false;
    })
  }
});