angular.module('starter.controllers.profile', [])

.controller('ProfileCtrl', function($scope, $rootScope, $http, $ionicModal, $ionicPopup, $ionicLoading, $timeout, PROFILE_EVENTS, AUTH_EVENTS, WorldMarkJourneyAPI, Profile) {
  $scope.loaded = false;
  $scope.editing = false;
  $scope.member = null;
  $scope.importantTypes = {
    important_school_travel: 'School Holiday travel',
    important_low_season: 'Low season travel',
    important_favourite_resort_availability: 'Favourite resort availability',
    important_last_minute_deals: 'Last minute deals'
  };

  $scope.changed = false;

  $scope.$watch('member', function(newValue, oldValue) {
    if(oldValue) $scope.changed = true;
  }, true);

  $scope.loadResults = function(pulledToRefresh) {
    $scope.loaded = false;
    $scope.errorMessage = null;

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Profile.get().$promise
    .then(function(response) {
      $scope.member = response;
      $scope.loaded = true;
      $timeout(function() {$scope.changed = false;});
    })
    .then($scope.loadActivities)
    .catch(function() {
      $scope.errorMessage = 'Error loading profile. Please try again';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.toggleEditing = function() {
    $scope.editing = !$scope.editing;
  }

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  });

  // Create the location selection modal that we will use later
  var locationSelectionScope = $scope.$new(true);
  $ionicModal.fromTemplateUrl('templates/location-selection-modal.html', {
    scope: locationSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.locationModal = modal;
  });

  locationSelectionScope.$on('locationselectionmodal.close', function(e) {
    $scope.locationModal.hide();
  });

  locationSelectionScope.$on('locationselectionmodal.complete', function(e, location) {
    $scope.member.home_location_name = location.name;
    $scope.member.home_location_lat = location.latitude;
    $scope.member.home_location_lon = location.longitude;
    $scope.locationModal.hide();
  });

  $scope.openLocationModal = function() {
    locationSelectionScope.$broadcast('locationselectionmodal.select', {
      name: $scope.member.home_location_name,
      latitude: Number($scope.member.home_location_lat),
      longitude: Number($scope.member.home_location_lon)
    });
    $scope.locationModal.show();
  }

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  resortSelectionScope.allowEmpty = true;
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.resortModal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.resortModal.hide();
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resorts) {
    $scope.member.favourite_resorts = resorts;
    $scope.resortModal.hide();
  });

  $scope.openResortModal = function() {
    resortSelectionScope.selection = angular.extend([], $scope.member.favourite_resorts);
    $scope.resortModal.show();
  }

  $scope.$on('$destroy', function() {
    $scope.locationModal.remove();
    delete locationSelectionScope;

    $scope.resortModal.remove();
    delete resortSelectionScope;
  });

  $scope.updateImportant = function(name) {
    var count = 0;
    angular.forEach($scope.importantTypes, function(label, type) {
      if($scope.member[type]) count++;
    });

    if(count > 2) {
      $scope.member[name] = false;
      $ionicPopup.alert({
        title: 'Maximum Options',
        template: 'A maximum of 2 options can be chosen. Please deselect an existing option first',
      });
    }
  }

  $scope.openWorldMarkLoginModal = function() {
    var worldmarkScope = $scope.$new();
    worldmarkScope.data = {};
    worldmarkScope.disconnect = function() {
      $scope.member.worldmark_journey_id = null;
      $scope.member.worldmark_journey_key = null;
    }
    $ionicPopup.prompt({
      title: 'WorldMark Journey Login',
      template: '<div class="text-center" ng-if="member.worldmark_journey_id"><a href="#" ng-click="disconnect()" class="button button-assertive button-clear">Disconnect</a></div><p>Please enter your WorldMark Journey password</p><input type="password" ng-model="data.password" placeholder="Password">',
      scope: worldmarkScope,
      buttons: [
        {
          text: 'Cancel',
          type: 'button-default'
        },
        {
          text: 'OK',
          type: 'button-positive',
          onTap: function(e) {
            return worldmarkScope.data.password;
          }
        }
      ]
    }).then(function(res) {
      if(res !== undefined) {
        $ionicLoading.show();
        $http.post(WorldMarkJourneyAPI.url('connect/'), {
          owner_number: $scope.member.owner_number,
          password: res
        }).then(function(result) {
          var response = result.data;
          if(response.success) {
            $scope.member.worldmark_journey_id = response.data.id;
            $scope.member.worldmark_journey_key = response.data.key;
          } else {
            $ionicPopup.alert({
              title: 'Error',
              template: (typeof response.data == 'string')? response.data : 'Unknown error. Please try again'
            }).then($scope.openWorldMarkLoginModal)
          }
        }).catch(function() {
          $ionicPopup.alert({
            title: 'Server Error',
            template: 'Error contacting server. Please try again later'
          }).then($scope.openWorldMarkLoginModal)
        }).finally(function() {
          $ionicLoading.hide();
        })
      } else {
        // Cancel was selected
      }
    });
  }

  $scope.save = function() {
    $ionicLoading.show();
    $scope.errorMessage = null;
    return Profile.save($scope.member).$promise
    .then(function() {
      $scope.changed = false;
      $rootScope.$broadcast(PROFILE_EVENTS.update, $scope.member);
    })
    .catch(function() {
      $scope.errorMessage = 'Error saving profile. Please try again';
    })
    .finally(function() {
      $ionicLoading.hide();
    });
  }

  // Trigger reload
  var setUnloaded = function(e) {
    $scope.loaded = false;
    $scope.member = null;
  }
  var unbindLogin = $rootScope.$on(AUTH_EVENTS.login, setUnloaded);
  $scope.$on('$destroy', unbindLogin);

  $scope.preventExitUnbind = null;

  // Prevent user from leaving
  $scope.$on('$ionicView.enter', function(e) {
    if($scope.preventExitUnbind) return;

    $scope.preventExitUnbind = $rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams) {
      if($scope.changed) {
        if(!toState.name.match(/^(app.profile|app.logout)/)) {
          event.preventDefault();
          $ionicPopup.alert({
            title: 'Profile Unsaved',
            template: 'You must save your profile first',
          });
        } else if(toState.name.match(/^app.logout/)) {
          $scope.preventExitUnbind();
          $scope.preventExitUnbind = null;
        }
      } else {
        $scope.preventExitUnbind();
        $scope.preventExitUnbind = null;
      }
    });
  });
})

.controller('ProfileBookingWindowsCtrl', function($scope, $ionicModal, $timeout, $ionicLoading, Profile) {
  $scope.loaded = false;
  $scope.changed = false;

  $scope.$watch('bookingWindows', function(newValue, oldValue) {
    if(oldValue) $scope.changed = true;
  }, true)

  $scope.removeBookingWindow = function(index) {
    $scope.bookingWindows.splice(index, 1);
  }

  $scope.addBookingWindow = function(index) {
    // Add 13 months to date
    var date = new Date();
    date.setMonth(date.getMonth() + 13);

    $scope.bookingWindows.push({
      start_date: date
    });
  }

  $scope.save = function() {
    $ionicLoading.show();
    $scope.errorMessage = null;
    return Profile.saveBookingWindows({booking_windows: $scope.bookingWindows}).$promise
    .then(function() {
      $scope.changed = false;
    })
    .catch(function() {
      $scope.errorMessage = 'Error saving booking windows. Please try again';
    })
    .finally(function() {
      $ionicLoading.hide();
    });
  }

  $scope.loadResults = function(pulledToRefresh) {
    $scope.loaded = false;
    $scope.errorMessage = null;

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Profile.bookingWindows().$promise
    .then(function(bookingWindows) {
      angular.forEach(bookingWindows, function(bookingWindow) {
        bookingWindow.start_date = moment(bookingWindow.start_date, 'YYYY-MM-DD HH:mm:ss').utc().toDate();
      })
      $scope.bookingWindows = bookingWindows;
      $timeout(function() {$scope.changed = false;});
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading booking windows. Please try again';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.$on('$ionicView.enter', function() {
    if(!$scope.loaded || $scope.errorMessage) {
      $scope.loadResults()
    }
  });

  // Create the resort selection modal that we will use later
  var resortSelectionScope = $scope.$new(true);
  resortSelectionScope.multiple = false;
  $ionicModal.fromTemplateUrl('templates/resort-selection-modal.html', {
    scope: resortSelectionScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  resortSelectionScope.$on('resortselectionmodal.close', function(e) {
    $scope.modal.hide();
    $scope.activeBookingWindow = null;
  });

  resortSelectionScope.$on('resortselectionmodal.complete', function(e, resort) {
    $scope.activeBookingWindow.resort_id = resort.record_id;
    $scope.activeBookingWindow.resort_name = resort.name;
    $scope.modal.hide();
  });

  $scope.openModal = function(bookingWindow) {
    $scope.activeBookingWindow = bookingWindow;
    $scope.modal.show();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete resortSelectionScope;
  });
})