angular.module('starter.controllers.resorts', [])

.controller('AbstractResortListingCtrl', function($scope, $stateParams, Resort) {
  $scope.title = $scope.title || 'Resorts';

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.resorts = null;
    $scope.loaded = false;

    return $scope.getResultsPromise()
    .then(function(resorts) {
      $scope.resorts = resorts;
      $scope.loaded = true;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading resorts';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.loadResults();
})

.controller('ResortListingCtrl', function($scope, $stateParams, $controller, Resort) {
  $scope.formatTitle = function(params) {
    if(params) {
      return params.country + (params.state? ' - ' + params.state : '');
    } else {
      return 'Resorts';
    }
  }

  $scope.title = $scope.formatTitle($stateParams);

  $scope.getResultsPromise = function() {
    return Resort.query($stateParams).$promise;
  }

  // Initialize the super class and extend it.
  angular.extend(this, $controller('AbstractResortListingCtrl', {$scope: $scope}));
})

.controller('ResortSearchCtrl', function($scope, $q, $filter, $log, Resort) {
  $scope.query = {
    keywords: null
  }

  $scope.reset = function() {
    $scope.errorMessage = null;
    $scope.regions = null;
  }

  $scope.search = ionic.debounce(function() {
    $log.log('Searching: ' + $scope.query.keywords);
    $scope.promise = $scope.loadResults();
  }, 500);

  $scope.clear = function() {
    $scope.query.keywords = null;
  }

  $scope.loadResults = function(pulledToRefresh) {
    if($scope.canceller) {
      $scope.canceller.resolve();// Aborts the $http request if it isn't finished.
    }
    $scope.canceller = $q.defer();
    $scope.reset();

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Resort.query($scope.query, {
      timeout: $scope.canceller.promise
    }).$promise
    .then(function(resorts) {
      $scope.resorts = resorts;
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading resorts';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.loadResults();
})

.controller('RegionCtrl', function($scope, $q, $filter, $log, Resort) {
  $scope.reset = function() {
    $scope.errorMessage = null;
    $scope.regions = null;
  }

  $scope.loadResults = function(pulledToRefresh) {
    $scope.reset();

    if(!pulledToRefresh) {
      $scope.refresherActive = false;
      $scope.$broadcast('loading.show');
    }

    return Resort.regions($scope.query)
    .$promise
    .then(function(regions) {
      $scope.regions = regions;
      $scope.featuredRegions = $filter('filter')(regions, {featured: true});
      $scope.standardRegions = $filter('filter')(regions, {featured: false});
    })
    .catch(function(message) {
      $scope.errorMessage = 'Error loading regions';
    })
    .finally(function() {
      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  $scope.loadResults();
})

.controller('ResortCtrl', function($scope, $window, $stateParams, $ionicNavBarDelegate, $ionicModal, $ionicSlideBoxDelegate, $ionicActionSheet, Resort, Config) {
  $scope.bookingUrl = Config.urls.booking;
  $scope.loading = true;
  $scope.title = 'Loading...';

  $scope.loadResults = function() {
    $scope.errorMessage = null;
    $scope.loading = true;
    $scope.$broadcast('loading.show');

    return Resort.get({resortId: $stateParams.resortId}).$promise
    .then(function(resort) {
      $scope.resort = resort;
      $scope.galleryImages = resort.images;
      $scope.title = resort.name;
    })
    .catch(function() {
      $scope.title = 'Error';
      $scope.errorMessage = 'Error loading resort. Please try again';
    })
    .finally(function() {
      $scope.loading = false;
      $scope.$broadcast('loading.hide');
    });
  }

  // Create the gallery modal that we will use later
  $ionicModal.fromTemplateUrl('templates/gallery-modal.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.galleryModal = modal;
  });

  $scope.$on('$destroy', function() {
    $scope.galleryModal.remove();
  });

  $scope.openGallery = function() {
    $ionicSlideBoxDelegate.slide(0);
    $scope.galleryModal.show();
  }

  $scope.closeGallery = function() {
    $scope.galleryModal.hide();
  }

  $scope.openPhoneList = function() {
    // Show the action sheet
    var hideSheet = $ionicActionSheet.show({
      buttons: [
        { text: '<b>AU</b> ' + $scope.resort.au_phone },
        { text: '<b>NZ</b> ' + $scope.resort.nz_phone },
        { text: '<b>Fiji</b> ' + $scope.resort.fiji_phone }
      ],
      titleText: 'Call ' + $scope.resort.name,
      cancelText: 'Cancel',
      cancel: function() {
        hideSheet();
      },
      buttonClicked: function(index) {
        var phoneNumbers = [$scope.resort.au_phone, $scope.resort.nz_phone, $scope.resort.fiji_phone];
        var number = phoneNumbers[index];

        if(number) {
          $window.open('tel:' + number.replace(/\s+/g, ''));
        }
      }
    });
  }

  // Load only once
  var unset = $scope.$on('$ionicView.enter', function() {
    $scope.loadResults();
    unset();
  });
})

.controller('ResortFacilitiesCtrl', function($scope, $stateParams, Resort) {
  $scope.reset = function() {
    $scope.errorMessage = null;
    $scope.resort = null;
    $scope.facilities = null;
    $scope.loading = {};
  }

  $scope.loadResults = function() {
    $scope.reset();
    $scope.loading.resort = true;
    $scope.loading.facilities = true;

    return Resort.get({resortId: $stateParams.resortId}).$promise
    .then(function(resort) {
      $scope.loading.resort = false;
      $scope.resort = resort;

      return Resort.facilities({resortId: resort.record_id}).$promise;
    })
    .then(function(facilities) {
      $scope.loading.facilities = false;
      $scope.facilities = facilities;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading details';
    })
    .finally(function() {
      $scope.loading.resort = false;
      $scope.loading.facilities = false;

      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  // Load only once
  var unset = $scope.$on('$ionicView.enter', function() {
    $scope.loadResults();
    unset();
  });
})

.controller('ResortRoomFacilitiesCtrl', function($scope, $stateParams, Resort) {
  $scope.reset = function() {
    $scope.errorMessage = null;
    $scope.resort = null;
    $scope.rooms = null;
    $scope.loading = {};
  }

  $scope.loadResults = function() {
    $scope.reset();
    $scope.loading.resort = true;
    $scope.loading.rooms = true;

    return Resort.get({resortId: $stateParams.resortId}).$promise
    .then(function(resort) {
      $scope.loading.resort = false;
      $scope.resort = resort;

      return Resort.roomFacilities({resortId: resort.record_id}).$promise;
    })
    .then(function(rooms) {
      $scope.loading.rooms = false;
      $scope.rooms = rooms;
    })
    .catch(function() {
      $scope.errorMessage = 'Error loading details';
    })
    .finally(function() {
      $scope.loading.resort = false;
      $scope.loading.rooms = false;

      // Refresher active needs to be false for initial load
      $scope.refresherActive = true;
      $scope.$broadcast('loading.hide');
      $scope.$broadcast('scroll.refreshComplete');
    });
  }

  // Load only once
  var unset = $scope.$on('$ionicView.enter', function() {
    $scope.loadResults();
    unset();
  });
})

.controller('ResortMapCtrl', function($scope, $ionicModal, $stateParams, $log, $q, $ionicPopup, $cordovaGeolocation, Resort, Activity) {
  $scope.map = {
    center: {
      latitude: -30.265086,
      longitude: 153.13719
    },
    bounds: {},
    zoom: 12,
    control: {}
  };

  if($stateParams.coords) {
    $scope.map.center.latitude = $stateParams.coords[0];
    $scope.map.center.longitude = $stateParams.coords[1];
  }

  $scope.activities = [];

  $scope.$on('$ionicView.enter', function() {
    $scope.visible = true;
    Resort.get({resortId: $stateParams.resortId}).$promise.then(function(resort) {
      $scope.resort = resort;
      if($scope.map.control.refresh) {
        $scope.map.control.refresh({
          latitude: $scope.resort.location_latitude,
          longitude: $scope.resort.location_longitude
        });
      } else {
        $scope.map.center.latitude = resort.location_latitude;
        $scope.map.center.longitude = resort.location_longitude;
      }
    })
    .then($scope.loadActivities);
  });

  $scope.$on('$ionicView.leave', function() {
    $scope.visible = false;
  });

  $scope.loadActivities = function() {
    return Activity.getBySiteId($scope.resort.getawaysp_site_id).success(function(activities) {
      $scope.activities = angular.forEach(activities || [], function(activity) {
        activity.coords = { latitude: activity.latitude_auto, longitude: activity.longitude_auto };

        var url = 'img/map-default.png'
        if(activity.images && activity.images[0] && activity.images[0].url_thumb) {
          url = activity.images[0].url_thumb;
        }
        activity.icon = { url: url, scaledSize: {width: 40, height: 40} };

        return activity;
      });
    });
  }

  $scope.goToResort = function() {
    if($scope.resort && $scope.map.control.refresh) {
      $scope.map.control.refresh({
        latitude: $scope.resort.location_latitude,
        longitude: $scope.resort.location_longitude
      });
      $scope.map.zoom = 18;
    }
  }

  $scope.goToMe = function() {
    $scope.loadingPosition = true;

    $cordovaGeolocation.getCurrentPosition({timeout:20000}).then(function(position) {
      $scope.map.control.refresh({
        latitude: position.coords.latitude,
        longitude: position.coords.longitude
      });
      $scope.map.zoom = 18;
    }, function() {
      $ionicPopup.alert({
        title: 'Error',
        template: 'Unable to determine your location'
      })
    }).finally(function() {
      $scope.loadingPosition = false;
    });
  }

  var activityScope = $scope.$new(true);
  // Create the activity modal that we will use later
  $ionicModal.fromTemplateUrl('templates/activity-modal.html', {
    scope: activityScope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });

  $scope.openModal = function(marker, event, activity) {
    activityScope.activity = activity;
    $scope.modal.show();
  }

  activityScope.close = function() {
    $scope.modal.hide();
  }

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
    delete activityScope;
  });

})

.controller('ResortEnquiryCtrl', function($scope, $state, $stateParams, $ionicHistory, $ionicPopup, $http, API, Resort) {
  $scope.enquiry = {};
  $scope.forms = {};

  // Prefill email address if available
  $scope.$on('$ionicView.enter', function() {
    if($scope.currentUser && $scope.currentUser.owner_number && !$scope.enquiry.owner_number) {
      $scope.enquiry.owner_number = $scope.currentUser.owner_number;
    }
    if($scope.currentUser && $scope.currentUser.email && !$scope.enquiry.email) {
      $scope.enquiry.email = $scope.currentUser.email;
    }
  });

  $scope.submitForm = function() {
    // Allow validation errors to show
    $scope.forms.enquiry.$setSubmitted();

    if($scope.forms.enquiry.$valid) {
      if($scope.sending) {
        return;
      }
      $scope.sending = true;

      return $http.post(API.url('resorts/' + $stateParams.resortId + '/enquire'), $scope.enquiry)
      .then(function(result) {
        $ionicPopup.alert({
          title: 'Email Sent',
          template: 'Thanks for your enquiry. We will respond as soon as possible.',
        }).then(function() {
          $scope.enquiry = {};
          $scope.forms.enquiry.$setPristine();

          if($ionicHistory.backView()) {
            $ionicHistory.goBack();
          }
        })
      })
      .catch(function(message) {
        $ionicPopup.alert({
          title: 'Error',
          template: 'Unable to send your enquiry. Please try again later.',
        })
      })
      .finally(function() {
        $scope.sending = false;
      });
    }
  }
})