angular.module('starter.services', ['angular-cache', 'ngResource'])
.service('Config', function() {
  return {
    urls: {
      booking: 'https://new.wyndhamvrap.com/owner',
      grabIt: 'http://grabit.worldmarksp.com/',
      payment: 'http://www.paywyn.com.au/'
    }
  }
})

.service('API', function() {
  // TODO: Update
  var base = 'https://worldmarkapp.com/api-v2/1/';
  // var base = 'http://sandbox.v2.reaswmsp.icmsdemo.com/api/v1/';
  return {
    base: base,
    url: function(url) {
      return base + url;
    }
  };
})

.service('GetawaySPAPI', function() {
  // TODO: Update
  var base = 'http://getawaysp.com/api/';
  // var base = 'http://sandbox.getawaysp.com/api/';
  return {
    base: base,
    url: function(url) {
      return base + url;
    }
  };
})

.service('WorldMarkJourneyAPI', function() {
  // TODO: Update
  var base = 'https://worldmarkjourney.com/api/';
  // var base = 'http://sandbox.myjourney.com/api/';
  return {
    base: base,
    url: function(url) {
      return base + url;
    }
  };
})

.constant('HOLIDAY_EVENTS', {
  create: 'wmsp:holiday:create',
  edit: 'wmsp:holiday:edit',
  delete: 'wmsp:holiday:delete',
})

.constant('AUTH_EVENTS', {
  login: 'wmsp:auth:login',
  logout: 'wmsp:auth:logout',
})

.constant('PROFILE_EVENTS', {
  update: 'wmsp:profile:update'
})

.service('AuthPayloadFormatter', function() {
  return {
    format: function(payload) {
      return {
        last_name: payload.last_name,
        member_id: payload.sub
      }
    }
  }
})

.service('PushNotificationHandler', function($rootScope, $ionicPopup, $state, $cordovaPushV5, $auth, AuthPayloadFormatter, DeviceService, Notification, UnreadNotificationService) {
  return {
    show: function(push) {
      var currentMemberId = null;
      if($auth.getToken()) {
        var payload = AuthPayloadFormatter.format($auth.getPayload());
        if(payload) {
          currentMemberId = payload.member_id;
        }
      }

      var custom = (push.additionalData && push.additionalData.custom)? push.additionalData.custom : push.additionalData;
      if(custom.member_id && custom.member_id != currentMemberId) {
        // Invalid member id (might have logged into another account)
        return;
      }

      var popupScope = $rootScope.$new(true);
      var device = DeviceService.get();

      Notification.get({
        notificationId: custom.notification_id,
        platform: device.platform,
        uuid: device.uuid
      }).$promise.then(function(notification) {
        popupScope.notification = notification;
        UnreadNotificationService.markRead(device, notification, false);
        popupScope.$broadcast('loading.hide');
      }, function() {
        popupScope.errorMessage = 'An unexpected error occurred';
        popupScope.$broadcast('loading.hide');
      })

      $ionicPopup.confirm({
        title: push.message,
        template: '<loading-directive></loading-directive><div class="error-bar" ng-if="errorMessage">{{errorMessage}}</div><div ng-if="notification.body" ng-bind-html="notification.body"></div>',
        okText: 'Show',
        cancelText: 'Close',
        scope: popupScope
      }).then(function(res) {
        popupScope.$destroy();
        delete popupScope;
        if(res) {
          var route = custom.route;
          if(route) {
            var match = route.match(/(.+)\((.*)\)$/);
            if(match) {
              try {
                var params = JSON.parse(match[2]);
                $state.go(match[1], params);
              } catch(e) {}
            } else {
              $state.go(route);
            }
          } else {
            $state.go('app.notifications');
          }
        }
      });
    }
  }
})

.service('PushNotificationPermission', function($q, $localStorage, $ionicPopup, $cordovaPushV5, $rootScope, PushNotificationHandler, DeviceService, Device) {
  var pushIntialized = false;
  var localStorage = $localStorage;
  return {
    request: function() {
      if(pushIntialized) return $q.when('already_initialized');
      if(!ionic.Platform.isIOS() && !ionic.Platform.isAndroid()) return $q.when('invalid_platform_for_push_notifications');

      var device = DeviceService.get();

      // Setup promise to ask for push notification permission
      if(localStorage.pushToken) {
        var requestPromise = $q.when(true);
      } else {
        // Request permission
        var requestPromise = $ionicPopup.confirm({
          title: 'Let Resort Assistant Send Push Notifications?',
          template: 'Push notifications allow you to receive the latest updates and great offers. Make the most of the app by allowing this feature.'
        }).then(function(res) {
          if(res) {
            return $q.resolve(true);
          } else {
            return $q.reject('no_interest');
          }
        });
      }

      // Initialize push notification
      return requestPromise
        .then(function() {
          return $cordovaPushV5.initialize({
            android: {
              // TODO: Update
              senderID: '783005249692'
            },
            ios: {
              badge: true,
              sound: true
            }
          })
        })
        .then($cordovaPushV5.register)
        .then(function(pushToken) {
          localStorage.pushToken = pushToken;

          // Register the device
          Device.registerPush(angular.extend({}, device, {
            push_token: pushToken
          }))

          $cordovaPushV5.onNotification();
          $cordovaPushV5.onError();

          $rootScope.$on('$cordovaPushV5:notificationReceived', function(e, notification) {
            PushNotificationHandler.show(notification);
          });

          $rootScope.$on('$cordovaPushV5:errorOccurred', function(e, error) {
            $ionicPopup.alert({
              title: 'Unknown Error',
              template: 'An unknown push notification error occurred'
            });
          });

          pushIntialized = true;
        }).catch(function(err) {
          if(err != 'no_interest') {
            $ionicPopup.alert({
              title: 'Unknown Error',
              template: 'Error setting up push notifications'
            })
          }
        });
    }
  }
})

/*
  LoginModalService.show((bool) Whether to show notice).then(function(userDidLogIn) {})
 */
.service('LoginModalService', function($rootScope, $ionicModal, $ionicPopup, $state, $q, $timeout) {
  var loginModalScope = $rootScope.$new(true);
  loginModalScope.deferred = null;

  // Create the login modal that we will use later
  $ionicModal.fromTemplateUrl('templates/login-modal.html', {
    scope: loginModalScope
  }).then(function(modal) {
    loginModalScope.modal = modal;
  });

  return {
    scope: loginModalScope,
    hide: function() {
      loginModalScope.modal.hide();

      if(loginModalScope.deferred) {
        loginModalScope.deferred.reject('closed');
        loginModalScope.deferred = null;
      }
    },
    show: function(showNotice) {
      if(loginModalScope.deferred) {
        loginModalScope.deferred.reject('closed');
        loginModalScope.deferred = null;
      }
      loginModalScope.deferred = $q.defer();

      loginModalScope.showNotice = !!showNotice;

      // Need to wait for next digest cycle if showing immediately
      $timeout(function() {
        loginModalScope.modal.show();
      });

      return loginModalScope.deferred.promise.then(function(result) {
        if(!result.profile_saved) {
          return $ionicPopup.confirm({
            title: 'Profile Setup',
            template: 'To make the most of Resort Assistant we recommend you setup your profile. Would you like to do this now?'
          }).then(function(res) {
            if(res) {
              $state.go('app.profile')
              return $q.reject('Profile setup');
            } else {
              return $q.resolve(true);
            }
          })
        }
      });
    }
  };
})

.factory('Resort', function($resource, $filter, API) {
  return $resource(API.url('resorts/:resortId'), {}, {
    recommend: {
      method: 'GET',
      url: API.url('resorts/recommend'),
      isArray: true
    },
    regions: {
      method: 'GET',
      url: API.url('resorts/regions'),
      isArray: true
    },
    facilities: {
      method: 'GET',
      url: API.url('resorts/:resortId/facilities'),
      isArray: true
    },
    roomTypes: {
      method: 'GET',
      url: API.url('resorts/:resortId/rooms'),
      isArray: true
    },
    roomCredits: {
      method: 'GET',
      url: API.url('resorts/:resortId/rooms/credits'),
      isArray: false
    },
    roomCreditsCompare: {
      method: 'GET',
      url: API.url('resorts/:resortId/rooms/credits/compare'),
      isArray: true
    },
    roomFacilities: {
      method: 'GET',
      url: API.url('resorts/:resortId/rooms/facilities'),
      isArray: true
    }
  });
})

// Note: requires device id
.factory('Notification', function($resource, $filter, API) {
  return $resource(API.url('notifications/:notificationId'), {}, {
    markRead: {
      method: 'POST',
      params: { notificationId: '@record_id' },
      url: API.url('notifications/:notificationId/read'),
    },
    count: {
      method: 'GET',
      url: API.url('notifications/count'),
    },
  });
})

.service('UnreadNotificationService', function($cordovaPushV5, Notification) {
  var service = this;
  service.count = {};

  service.countForResort = function(resortId) {
    return (service.count && service.count.resorts)? service.count.resorts[resortId] : 0;
  }

  service.markRead = function(device, notification, http) {
    service.count.all = Math.max(service.count.all - 1, 0);
    $cordovaPushV5.setBadgeNumber(service.count.all);

    if(notification.resort_id && service.count.resorts) {
      service.count.resorts[notification.resort_id] = Math.max(service.count.resorts[notification.resort_id] - 1, 0);
    }
    notification.read_at = new Date();

    if(http === undefined || http) {
      Notification.markRead({
        platform: device.platform,
        uuid: device.uuid,
        record_id: notification.record_id
      });
    }
  }

  service.update = function(device) {
    Notification.count({
      platform: device.platform,
      uuid: device.uuid
    }).$promise.then(function(response) {
      service.count = response;
      $cordovaPushV5.setBadgeNumber(service.count.all);
    });
  }
  return service;
})

.service('DeviceService', function($ionicPlatform, $cordovaDevice, Device) {
  var device = null;

  return {
    setup: function() {
      return $ionicPlatform.ready(function() {
        device = window.device? $cordovaDevice.getDevice() : {
          model: 'computer',
          platform: 'web',
          uuid: '1234567890',
          version: 'unknown'
        };
        // Force platform to lower case
        device.platform = device.platform.toLowerCase();

        Device.register(device);
      }).then(function() {
        return device;
      });
    },
    get: function() {
      return device;
    }
  }
})

.factory('Device', function($resource, $filter, API) {
  return $resource(API.url('devices/:deviceId'), {}, {
    register: {
      method: 'POST',
      url: API.url('devices/register')
    },
    registerPush: {
      method: 'POST',
      url: API.url('devices/register/push')
    }
  });
})

.factory('Facility', function($resource, API) {
  return $resource(API.url('facilities/:facilityId'), {});
})

.factory('Page', function($resource, API) {
  return $resource(API.url('pages/:slug'), {});
})

.factory('Holiday', function($resource, API) {
  return $resource(API.url('holidays/:holidayId'), {}, {
    'delete': {
      method: 'DELETE',
      params: { holidayId: '@record_id' }
    },
    create: {
      method: 'POST'
    },
    save: {
      method: 'PUT',
      params: { holidayId: '@record_id' }
    },
    shared: {
      method: 'PUT',
      params: { holidayId: '@record_id' },
      url: API.url('holidays/:holidayId/shared')
    },
    // Use as a class action. e.g. Holiday.addActivitys({holidayId:123, activityIds:234}, {})
    // NOTE: Can support multiple ids (if comma separated string)
    addActivity: {
      method: 'POST',
      url: API.url('holidays/:holidayId/activities/:activityIds'),
      params: { holidayId: '@record_id' }
    },
    // Use as a class action. e.g. Holiday.removeActivity({holidayId:123, activityId:234}, {})
    removeActivity: {
      method: 'DELETE',
      url: API.url('holidays/:holidayId/activities/:activityId'),
      params: { holidayId: '@record_id' }
    }
  });
})

.factory('Profile', function($resource, API) {
  return $resource(API.url('profile'), {}, {
    'pilot': {
      method: 'GET',
      url: API.url('profile'),
      timeout: 5000
    },
    'get': {
      method: 'GET',
      url: API.url('profile')
    },
    'save': {
      method: 'POST',
      url: API.url('profile')
    },
    'bookingWindows': {
      method: 'GET',
      url: API.url('profile/booking-windows'),
      isArray: true
    },
    'saveBookingWindows': {
      method: 'POST',
      url: API.url('profile/booking-windows')
    }
  });
})

.factory('Article', function($resource, API) {
  return $resource(API.url('articles/:articleId'), {});
})

.factory('PostcardTemplate', function($resource, API) {
  return $resource(API.url('postcardtemplates/:articleId'), {});
})

.factory('PostcardData', function() {
  var details = {};
  return {
    merge: function(properties) {
      angular.extend(details, properties);
    },
    reset: function() {
      angular.forEach(details, function(value, key) {
        delete details[key];
      })
    },
    get: function() {
      return angular.extend({}, details);
    }
  };
})

.factory('Activity', function($http, GetawaySPAPI) {
  return {
    getBySiteId: function(siteId) {
      return $http.get(GetawaySPAPI.url('all/search'), {
        params: {
          site_id: siteId
        },
        skipAuthorization: true
      })
    },
    getByIds: function(activityIds) {
      return $http.get(GetawaySPAPI.url('all/list_specific'), {
        params: {
          ids: (activityIds || []).join(',')
        },
        skipAuthorization: true
      })
    }
  }
})

.factory('ItineraryTemplate', function($http, GetawaySPAPI) {
  return {
    getBySiteId: function(siteId) {
      return $http.get(GetawaySPAPI.url('itinerary_templates/search'), {
        params: {
          site_id: siteId
        },
        skipAuthorization: true
      })
    }
  }
})